var HelixWebServicesClient = require('../helix_web_services_client');

require('../polyfill');

describe('HelixWebServicesClient', function() {

  describe('logIn', function() {
    it('can log in and out with real creds', function(done) {

      var client = new HelixWebServicesClient({
        url: 'http://localhost:9000',
        user: 'jdoe'
      });

      client.logIn('johndoe1A!')
        .done(function() {
          expect(client.session_token).toBeDefined();

          client.logOut()
            .done(done)
            .fail(function() {
              done.fail("logout failed")
            });
        })
        .fail(done.fail);
    });

    it('will fail if you provide incorrect creds', function(done) {

      var client = new HelixWebServicesClient({
        url: 'http://localhost:9000',
        user: 'jdoe'
      });

      client.logIn('invalid')
        .done(done.fail)
        .fail(function(jqXHR) {
          expect(jqXHR.status).toBe(401);
          done();
        });
    });
  });

  // TODO we very likely want to actually create a test project
  describe('syncProjects', function() {

    var randomStr = require('crypto').randomBytes(8).toString('hex');
    var testProjectName = 'Test Project ' + randomStr;
    var testProjectId = 'test_project_' + randomStr;

    it('can create a sync project', function(done) {

      var client = new HelixWebServicesClient({
        url: 'http://localhost:9000',
        user: 'jdoe'
      });

      client.logIn('johndoe1A!')
        .fail(done.fail)
        .done(function() {
          client.createSyncProject({ name: testProjectName })
            .fail(function() {
              console.log('create project failed', arguments);
              done.fail('create project failed');
            })
            .done(function() {
              client.logOut().fail(done.fail).done(done);
            });
        });
    });

    it('can find our project in all projects on the server', function(done) {
      var client = new HelixWebServicesClient({
        url: 'http://localhost:9000',
        user: 'jdoe'
      });

      client.logIn('johndoe1A!')
        .fail(done.fail)
        .done(function() {
          client.syncProjects()
            .fail(done.fail)
            .done(function(projects) {
              var project;
              for (var ii = 0; ii < projects.length; ii++) {
                if (projects[ii].id == testProjectId)
                  project = projects[ii];
              }
              expect(project).toBeDefined();
              client.logOut().fail(done.fail).done(done);
            });
        });
    });
  });

  describe('listFiles', function() {

    it('should list depots if no path is specified', function(done) {
      var client = new HelixWebServicesClient({
        url: 'http://localhost:9000',
        user: 'jdoe'
      });

      client.logIn('johndoe1A!')
        .fail(done.fail)
        .done(function() {
          client.listFiles()
            .fail(done.fail)
            .done(function(items) {
              var depot = items.find(function(x) { return x.name == 'depot'; });
              expect(depot).toBeDefined();
              client.logOut().fail(done.fail).done(done);
            });
        });
    });

    it('can list directories in the "depot" depot', function(done) {
      var client = new HelixWebServicesClient({
        url: 'http://localhost:9000',
        user: 'jdoe'
      });

      client.logIn('johndoe1A!')
        .fail(done.fail)
        .done(function() {
          client.listFiles('depot')
            .fail(done.fail)
            .done(function(items) {
              var dev = items.find(function(x) { return x.name == 'dev'; });
              expect(dev).toBeDefined();
              client.logOut().fail(done.fail).done(done);
            });
        });
    });

    it('can list files in the "//depot/dev/Experimental" directory', function(done) {
      var client = new HelixWebServicesClient({
        url: 'http://localhost:9000',
        user: 'jdoe'
      });

      client.logIn('johndoe1A!')
        .fail(done.fail)
        .done(function() {
          client.listFiles('//depot/dev/Experimental')
            .fail(done.fail)
            .done(function(items) {
              var dev = items.find(function(x) { return x.name == 'README'; });
              expect(dev).toBeDefined();
              client.logOut().fail(done.fail).done(done);
            });
        });
    });
  });
});